// 7 Segment Common Anode Clock using 8 38mmx55mm LED 7
// segment characters
// Last Updated: 06-12-19
// Displays Time, Date, Temperature, Humidity, and Pressure, and chimes
// Wifi connection to allow update of time and date and receive current values of
// time,date, temperature, pressure, and humidity from clock.

#include <Wire.h>

#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27,16,2);

#include "SoftwareSerial.h"
SoftwareSerial mySoftwareSerial(D6, D7); // RX, TX

#include "DFRobotDFPlayerMini.h"
DFRobotDFPlayerMini mp3;

#include <SparkFunBME280.h>
#define SEALEVELPRESSURE_HPA (1013.25)
BME280 bme; //Uses I2C address 0x76 (jumper closed)

#include "RTClib.h"
RTC_DS3231 rtc;

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>

//Station mode you should put your ssid and password
const char *ssid = "WifiClock";
const char *password = "password"; 

// the IP address for the shield:
IPAddress ip(192, 168, 4, 1);  

ESP8266WebServer server(80);

void handleRoot() {
  server.send(200, "text/html", "<h1>Wifi Clock Ready</h1>");
}

int rows = 0;
int columns = 0;
int latchPin = D3; //pin 12 (Yellow)on the 595
int dataPin = D4; //pin 14 (Orange) on the 595
int clockPin = D5; //pin 11 (Green) on the 595

int dataArray[] = {192,249,164,176,153,146,130,248,128,152,127,156,173,163,255,137,251,247,239,191,142,198,140};
//D1 = 256, D2 = 512, D3 = 1024, D4 = 2048, D5 = 4096, D6 = 8192, D7 = 16384, D8 = 32768

int offset;
int test;
int days, months, years;
int new_days, new_months, new_years;
int hours, minutes, seconds;
int new_hours, new_minutes, new_seconds;
int found, maxIndex;
int digits_entered = 0;
int j,x,y,k,m,n,i,t,a,ledIndex;
int64_t index1;
int chimesVolumn = 20;
int ldrStatus;  // Ambiant light check, less than 700 clock switches off
int ldrLimit = 700;  
int sleep_mode = 0;

char testChar,indicator;
char ledCharacter[1];

String screenSaveRequired;
String chimesRequired = "Y";
String chimesPlayed = "N";
String c_check,hour_check;
String new_value_1, new_value_2;
String dayString;
String monthString;
String yearString;
String day1,day2;
String month1,month2;
String year1,year2,year3, year4;
String hourString;
String minuteString;
String secondString;
String hour1;
String minute1;
String second1;
String hour2;
String minute2;
String second2,save_second2;
String year_string,date_string;
String timeString, dateString;
String characterArray[7]; // Character Array
String humidityString;
String humidityString1;
String humidityString2;
String pressureString;
String pressureString1;
String pressureString2;
String temperatureCString;
String temperatureCString1;
String temperatureCString2;
String temperatureFString;
String temperatureFString1;
String temperatureFString2;
String header;
String messageText, para1,para2;
String update_days,update_months,update_years,update_hours,update_minutes,update_seconds;

float pressure,temperatureC, temperatureF, humidity;

DateTime now;

void performGetCommand()
{ 
      // Format http://192.168.4.1/GET

      // Must not send a space, replace all spaces with 0.
      if (hour1 == " ") hour1 = "0";
      if (minute1 == " ") minute1 = "0";
      if (day1 == " ") day1 = "0";
      if (month1 == " ") month1 = "0";
      
      messageText = hour1 + hour2 + "," + minute1+minute2 + "," + 
                    second1+second2 + "," + day1 + day2 + "," + month1 + month2 + ",20" + 
                    year3 + year4 + "," + humidityString1 + "," + humidityString2 + "," +
                    pressureString1 + "," + pressureString2 + "," + temperatureCString1 + "," +
                    temperatureCString2 + "," + temperatureFString1 + "," + temperatureFString2;
      server.send(200, "text/plain",messageText);
      Serial.println(messageText);
         
      delay(10);  
}
void performSetCommand()
{
      // FORMAT  http://192.168.4.1/SET?PARA1=HH-MM-SS&PARA2=DD-MM-YY
          
      para1 = server.arg("PARA1");       
      para2 = server.arg("PARA2"); 
      update_hours = para1.substring(0,2);
      update_minutes = para1.substring(3,5);
      update_seconds = para1.substring(6,8);
      update_days = para2.substring(0,2);
      update_months = para2.substring(3,5);
      update_years = para2.substring(6,10); 
      new_days = update_days.toInt();
      new_months = update_months.toInt();
      new_years = update_years.toInt();
      new_hours = update_hours.toInt();
      new_minutes = update_minutes.toInt();
      new_seconds = update_seconds.toInt();  
      rtc.adjust(DateTime(new_years, new_months, new_days, new_hours, new_minutes, new_seconds)); 
      messageText = "Clock Updated";
      server.send(200, "text/plain",messageText);        
      delay(10);
}
void setup()
{  
  
  // Initialize the output variables as outputs
  pinMode(latchPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  
  Serial.begin(115200);

  // Setup WiFi Access Point on WEMOS
  WiFi.mode(WIFI_AP);
  WiFi.softAP(ssid, password, 1);
  IPAddress myIP = WiFi.softAPIP();
  server.on("/", handleRoot);
  server.begin();

  server.on("/GET", performGetCommand);
  server.on("/SET", performSetCommand);
  
  // initialize the lcd 
  lcd.init();
  lcd.backlight();
  rtc.begin(); // Initialize the rtc object
//  rtc.adjust(DateTime(2019, 11, 12, 9, 52, 0)); 
  Wire.begin();
  bme.setI2CAddress(0x76); //Connect to a second sensor 0x76 or 0x77
  if(bme.beginI2C() == false) 
  {
    Serial.println("Sensor B connect failed"); 
  } 
  indicator = 'W'; // first part of rolling time seperator
  sleep_mode = 0;
  screenSaveRequired = "Y";
  chimesPlayed = "N";
  // setup chimes
  mySoftwareSerial.begin(9600);
  mp3.begin(mySoftwareSerial);
  mp3.reset();  
  mp3.volume(30);  //Set volume value. From 0 to 30
  
  lcd.setCursor(0,0);
  lcd.print("Segment Clock V2"); 
  ldrStatus = analogRead(A0);
//  ldrStatus = ldrStatus * 150;  // Adjustment for difference between this clock and the original clock
                                // this will ensure that both clocks switch off approximately at the same time.                           
Serial.println(ldrStatus);
}    
void loop()
{   
  server.handleClient();
  light_detection(); // Check that light levels are OK, if dark then switch off Nixie tubes until
                    // light levels return to full.                  
  if (sleep_mode != 1)
  { 
    DateTime now = rtc.now();
    // Display time for 30 seconds  
    display_time();
    lcd_display(); 
    
    // Check for Chime
    if (chimesRequired == "Y" and chimesPlayed == "N")
          {
              c_check = characterArray[3] + characterArray[4]; 
              if (c_check == "00" or c_check == "15" or c_check == "30" or c_check == "45")
                  {
                      play_chimes();                  
                      chimesPlayed = "Y"; 
                  }
          }     
    for (t = 1; t <= 3200; t++) // equates to a display time of 45 seconds  = 3200
    {    
          display_time();  // Display time on VFD screen 
          server.handleClient();      
    }

    chimesPlayed = "N";
    
    // Display date    
    dayString = now.day();
    display_date();  // Display date on VFD screen
    lcd_display();
    for (t = 1; t <= 300; t++) // equates to a display time of 5 seconds = 400
    {         
          display_date();  // Display date on VFD screen
          server.handleClient();
    }    
    // Display temperature in Celisus
    temperatureC = bme.readTempC();   
    display_temperatureC();  // Display temperatureC on VFD screen  
    lcd_display(); 
    for (t = 1; t <= 250; t++) 
    {      
          display_temperatureC();  // Display temperatureC on VFD screen           
          server.handleClient();
    }
    // Display temperature in Farenheit
    temperatureF = bme.readTempF();   
    display_temperatureF();  // Display temperatureF on VFD screen 
    lcd_display();
    for (t = 1; t <= 250; t++) 
    {            
          display_temperatureF();  // Display temperatureF on VFD screen         
          server.handleClient();
    }
    // Display pressure in mB
    pressure = bme.readFloatPressure()/100.0F;  
    display_pressure();  // Display pressure on VFD screen  
    lcd_display();    
    for (t = 1; t <= 250; t++)
    {
          display_pressure();  // Display pressure on VFD screen                  
          server.handleClient();
    }
    // Display humidity
    humidity = bme.readFloatHumidity();
    display_humidity();  // Display humidity on VFD screen       
    lcd_display();
    for (t = 1; t <= 250; t++) 
    {       
          display_humidity();  // Display humidity on VFD screen         
          server.handleClient();
    }
  }        
}

void light_detection()
{
ldrStatus = analogRead(A0);
//ldrStatus = ldrStatus * 150;  // Adjustment for difference between this clock and the original clock
                             // this will ensure that both clocks switch off approximately at the same time.
Serial.println(ldrStatus);
sleep_mode = 0;
if (ldrStatus <= ldrLimit and screenSaveRequired == "Y")
  {
   sleep_mode = 1;
  characterArray[0] = " "; // Setup Blank Screen
  characterArray[1] = " "; 
  characterArray[2] = " ";
  characterArray[3] = " "; 
  characterArray[4] = " "; 
  characterArray[5] = " ";    
  characterArray[6] = " ";
  characterArray[7] = " "; 
  lcd.setCursor(0,1);
  lcd.print(" SLEEP MODE ");
  lcd.print(ldrStatus); 
  process_display();  
  }
}
void process_chime_display()
{
  characterArray[0] = " "; // Chime Screen
  characterArray[1] = " "; 
  characterArray[2] = " ";
  characterArray[3] = " "; 
  characterArray[4] = " "; 
  characterArray[5] = " ";    
  characterArray[6] = " ";
  characterArray[7] = "."; 
  process_display();  
}
void play_chimes()
{
   if (c_check == "00")  // Sound hourly chime
   {          
        process_chime_display();
        mp3.play(16);
        delay(19000);  
        if (hour_check == " 1" or hour_check == "13")
        {
          // Play 1 Hour Chime
        mp3.play(1);
        delay(3000);           
        }
        if (hour_check == " 2" or hour_check == "14")
        {
          // Play 2 Hour Chime
        mp3.play(2);
        delay(5000);           
        }
        if (hour_check == " 3" or hour_check == "15")
        {         
          // Play 3 Hour Chime
        mp3.play(3);  
        delay(7000);                 
        }
        if (hour_check == " 4" or hour_check == "16")
        {        
          // Play 4 Hour Chime
        mp3.play(4); 
        delay(9000);                   
        }
        if (hour_check == " 5" or hour_check == "17")
        {          
          // Play 5 Hour Chime 
        mp3.play(5);  
        delay(11000);                
        }
        if (hour_check == " 6" or hour_check == "18")
        {         
          // Play 6 Hour Chime
        mp3.play(6);
        delay(13000);                     
        } 
        if (hour_check == " 7" or hour_check == "19")
        {         
          // Play 7 Hour Chime
        mp3.play(7);         
        delay(15000);   
        }
        if (hour_check == " 8" or hour_check == "20")
        {     
          // Play 8 Hour Chime
        mp3.play(8);   
        delay(17000);               
        }
        if (hour_check == " 9" or hour_check == "21")
        {       
          // Play 9 Hour Chime 
        mp3.play(9);         
        delay(19000); 
        }
        if (hour_check == "10" or hour_check == "22")
        {        
          // Play 10 Hour Chime 
        mp3.play(10);       
        delay(21000); 
        }
        if (hour_check == "11" or hour_check == "23")
        {         
          // Play 11 Hour Chime 
        mp3.play(11);
        delay(23000);       
        }
        if (hour_check == "12" or hour_check == "00")
        {        
          // Play 12 Hour Chime
        mp3.play(12); 
        delay(25000);    
        } 
   } 
   else {
             process_chime_display();   
             if (c_check == "15")  // Sound 15 minute chime
             {
              // play 15 Minute chime 
             mp3.play(13);          
             }
             if (c_check == "30")  // Sound 30 minute chime
             {
              // play 30 Minute chime            
             mp3.play(14);   
             }
              if (c_check == "45")  // Sound 45 minute chime
             {             
              // play 45 Minute chime  
             mp3.play(15);   
             }
          }     
}

void lcd_display() // Display on the LCD exactly what is stored in the character array.
{
  lcd.setCursor(0,1);
  lcd.print("               ");
  lcd.setCursor(0,1);
  lcd.print(characterArray[0]);
  lcd.print(characterArray[1]);
  if (characterArray[2] == "W" or characterArray[2] == "X" or characterArray[2] == "Y" or characterArray[2] == "Z")
  {
    lcd.print(".");
  } else
     { 
      lcd.print(characterArray[2]);
     }
  lcd.print(characterArray[3]);
  lcd.print(characterArray[4]);
  if (characterArray[2] == "W" or characterArray[2] == "X" or characterArray[2] == "Y" or characterArray[2] == "Z")
  {
    lcd.print(".");
  } else
     { 
      lcd.print(characterArray[5]);
     }
  if (characterArray[6] == "U")
  {
    lcd.print("o");
  } else
     { 
      lcd.print(characterArray[6]);
     }     
  if (characterArray[7] == "L")
  {
    lcd.print("o");
  } else
     { 
      lcd.print(characterArray[7]);
     } 
}

void display_time()
{
  now = rtc.now();
  hourString = now.hour();
  test = hourString.length();
  if (test == 1)
    {
          hour1 = " ";
          hour2 = hourString.substring(0,1);       
    }
  else {
          hour1 = hourString.substring(0,1);
          hour2 = hourString.substring(1,2);  
       }  
  minuteString = now.minute();
  test = minuteString.length();
  if (test == 1)
    {
          minute1 = "0";
          minute2 = minuteString.substring(0,1);       
    }
  else {
          minute1 = minuteString.substring(0,1);
          minute2 = minuteString.substring(1,2);  
       }
  secondString = now.second();
  test = secondString.length();
  if (test == 1)
    {
          second1 = "0";
          second2 = secondString.substring(0,1);       
    }
  else {
          second1 = secondString.substring(0,1);
          second2 = secondString.substring(1,2); 
       }
      
  characterArray[0] = hour1;
  characterArray[1] = hour2;
  hour_check = characterArray[0]  + characterArray[1];   
  characterArray[2] = indicator;
  characterArray[3] = minute1; 
  characterArray[4] = minute2; 
  characterArray[5] = indicator;   
  characterArray[6] = second1; 
  characterArray[7] = second2;  

  if(second2 != save_second2)  // Move the indicator one position at the change of each second
  {
    if (indicator == 'W')
    {
      indicator = 'X';
    } else if (indicator == 'X')
           {
             indicator = 'Y';
           } else if (indicator == 'Y')
                  {
                    indicator = 'Z';
                  } else {
                           indicator = 'W';
                         }
    save_second2 = second2;                     
  }
  process_display();       
}

void display_date()
{
  now = rtc.now();
  dayString = now.day();
  test = dayString.length();
  if (test == 1)
    {
          day1 = " ";
          day2 = dayString.substring(0,1);       
    }
  else {
          day1 = dayString.substring(0,1);
          day2 = dayString.substring(1,2);  
       }
  monthString = now.month();
  test = monthString.length();
  if (test == 1)
    {
          month1 = "0";
          month2 = monthString.substring(0,1);       
    }
  else {
          month1 = monthString.substring(0,1);
          month2 = monthString.substring(1,2);  
       }
  yearString = now.year();
  year1 = yearString.substring(0,1);
  year2 = yearString.substring(1,2); 
  year3 = yearString.substring(2,3);
  year4 = yearString.substring(3,4); 
  characterArray[0] = day1;  
  characterArray[1] = day2;
  characterArray[2] = "W";
  characterArray[3] = month1; 
  characterArray[4] = month2;
  characterArray[5] = "W"; 
  characterArray[6] = year3;
  characterArray[7] = year4;
  process_display();
}

void display_temperatureC()
{ 
  temperatureCString = String(temperatureC);  
  temperatureCString1 = getValue(temperatureCString, '.', 0); // 99.
  temperatureCString2 = getValue(temperatureCString, '.', 1); // .99
  characterArray[0] = " ";   
  characterArray[1] = temperatureCString1.substring(0,1);
  characterArray[2] = temperatureCString1.substring(1,2);  
  characterArray[3] = ".";
  characterArray[4] = temperatureCString2.substring(0,1); 
  characterArray[5] = temperatureCString2.substring(1,2);
  characterArray[6] = "U";
  characterArray[7] = "C"; 
  process_display();
}

void display_temperatureF()
{ 
  temperatureFString = String(temperatureF);  
  temperatureFString1 = getValue(temperatureFString, '.', 0); // 99.
  temperatureFString2 = getValue(temperatureFString, '.', 1); // .99
  characterArray[0] = " ";   
  characterArray[1] = temperatureFString1.substring(0,1);
  characterArray[2] = temperatureFString1.substring(1,2);  
  characterArray[3] = ".";
  characterArray[4] = temperatureFString2.substring(0,1); 
  characterArray[5] = temperatureFString2.substring(1,2);
  characterArray[6] = "U";
  characterArray[7] = "F"; 
  process_display();  
}

void display_pressure()
{
  pressureString = String(pressure);
  pressureString1 = getValue(pressureString, '.', 0); // 9999.
  pressureString2 = getValue(pressureString, '.', 1); // .99
  if (pressureString1.substring(0,1) == "9")  // correctly display pressure values less than 1000 mB
    {
      characterArray[0] = " ";
      characterArray[1] = pressureString1.substring(0,1); 
      characterArray[2] = pressureString1.substring(1,2);
      characterArray[3] = pressureString1.substring(2,3); 
    }
  else
    {  
      characterArray[0] = pressureString1.substring(0,1); 
      characterArray[1] = pressureString1.substring(1,2);
      characterArray[2] = pressureString1.substring(2,3); 
      characterArray[3] = pressureString1.substring(3,4); 
    }  
  characterArray[4] = ".";  
  characterArray[5] = pressureString2.substring(0,1);  
  characterArray[6] = pressureString2.substring(1,2); 
  characterArray[7] = "P";    
  process_display(); 
}

void display_humidity()
{
  humidityString = String(humidity);
  humidityString1 = getValue(humidityString, '.', 0); // 99.
  humidityString2 = getValue(humidityString, '.', 1); // .99
  characterArray[0] = humidityString2.substring(0,1); 
  characterArray[1] = humidityString2.substring(1,2); 
  characterArray[2] = ".";
  characterArray[3] = humidityString1.substring(0,1);
  characterArray[4] = humidityString1.substring(1,2);
  characterArray[5] = "L";
  characterArray[6] = "/";  
  characterArray[7] = "U";  
  process_display(); 
}

String getValue(String dataPin, char separator, int index2)
{
    found = 0;
    int strIndex[] = { 0, -1 };
    maxIndex = dataPin.length() - 1;
    for (i = 0; i <= maxIndex && found <= index2; i++) 
    {
        if (dataPin.charAt(i) == separator || i == maxIndex) 
        {
            found++;
            strIndex[0] = strIndex[1] + 1;
            strIndex[1] = (i == maxIndex) ? i+1 : i;
        }
    }
    return found > index2 ? dataPin.substring(strIndex[0], strIndex[1]) : "";
}

void write_character()
{
    index1 = 0;
   
  if(characterArray[offset] == "0")
    index1 = 0;
  if(characterArray[offset] == "1")
    index1 = 1;
  if(characterArray[offset] == "2")
    index1 = 2;
  if(characterArray[offset] == "3")
    index1 = 3;
  if(characterArray[offset] == "4")
    index1 = 4;
  if(characterArray[offset] == "5")
    index1 = 5;
  if(characterArray[offset] == "6")
    index1 = 6;                        
  if(characterArray[offset] == "7")
    index1 = 7;
  if(characterArray[offset] == "8")
    index1 = 8;
  if(characterArray[offset] == "9")
    index1 = 9;     
  if(characterArray[offset] == ".")
    index1 = 10;
  if(characterArray[offset] == "U")  // Upper Zero
    index1 = 11;
  if(characterArray[offset] == "/")
    index1 = 12;
  if(characterArray[offset] == "L")  // Lower Zero
    index1 = 13;
  if(characterArray[offset] == " ")
    index1 = 14;
  if(characterArray[offset] == "H")
    index1 = 15;                        
  if(characterArray[offset] == "X")
    index1 = 16;
  if(characterArray[offset] == "Y")
    index1 = 17;
  if(characterArray[offset] == "Z")
    index1 = 18;
  if(characterArray[offset] == "W")
    index1 = 19;                  
  if(characterArray[offset] == "F")
    index1 = 20; 
  if(characterArray[offset] == "C")
    index1 = 21; 
  if(characterArray[offset] == "P")
    index1 = 22;  
}

void process_display()
{
  ledIndex = 256;
  for (offset=0; offset<=7; offset++)
  {
    write_character();
    digitalWrite(latchPin, LOW);
    shiftOut(dataPin, clockPin, MSBFIRST, ledIndex + 64 >> 8);
    shiftOut(dataPin, clockPin, MSBFIRST, ledIndex + dataArray[index1]);
    digitalWrite(latchPin, HIGH);
    ledIndex = ledIndex * 2;
    delay(1);
  }
}
